######################################
# File with Functions and Structures #
######################################

# Structure for parameters
mutable struct ParamsStruc{T <: Real}
    # Discount factor
    β::T
    # Annual Growth Rate Vector
    μ_baseline::T           # Baseline Gross Growth Rate
    μ_vec::AbstractArray    # Gross Growth Rate Vector
    # Equivalent Variation
    ε_EV::Vector{T}
    # Compensating Variation
    ε_CV::Vector{T}
end

################################################################################

# Set parameter values
function SetParams(; β=0.95, μ=3.0)
    # Baseline gross growth rate
    μ_baseline = 1.0 + μ/100.0
    # Gross Growth Rate Vector
    μ_vec = 1.0 .+ range(1.0,step=0.5,stop=6.0)./100
    # Equivalent Variation
    ε_EV  = exp.( β/(1.0-β) .* (log.(μ_vec).-log(μ_baseline)) ) .- 1.0
    # Compensating Variation
    ε_CV = exp.( β/(1.0-β) .* (log(μ_baseline).-log.(μ_vec)) ) .- 1.0
    # Output
    ParamsStruc(β,μ_baseline,μ_vec,ε_EV,ε_CV)
end

################################################################################

# Output table with key statistics
function OutputTable(Gr,EV,CV)
    # Table header
    table_header = ["Growth rate", "Equivalent Variation", "Compensating Variation"]
    table_data = [Gr EV CV]
    # Print tables
    print("################################################################\n")
    pretty_table(table_data; header = table_header, tf = tf_markdown, formatters = ft_printf("%5.2f", 2:3) )
    print("Note for Table: Numbers are Percentages\n")
    # Save table as dataframe
    df = DataFrame("Growth rate" => Gr, "Equivalent Variation" => EV,
                    "Compensating Variation" => CV)
    return df
end

################################################################################
