######################################
# File with Functions and Structures #
######################################

# Structure for parameters
mutable struct ParamsStruc{T <: Real}
    # Demand curve
    α::T        # constant
    β::T        # slope
    # Cost function
    γ::T        # quadratic term
    # Grid of output
    ogrid::AbstractArray
end

################################################################################

# Set parameter values
function SetParams(; α = 1.0, β = 0.5, γ = 0.5)
    # Construct grid of output points
    ogrid = range(0.0,step=α/(β*100.0),stop=α/β)
    # Output
    ParamsStruc(α,β,γ,ogrid)
end

################################################################################

# Monopolist's Marginal cost curve
# Gives marginal cost as a function of output
function MarginalCost(output,Params)
    # Unpack parameters
    @unpack γ = Params
    # Margina cost
    return γ.*output
end

################################################################################

# Monopolist's marginal revenue curve
# Gives marginal revenue as a function of output
function MarginalRevenue(output,Params)
    # Unpack parameters
    @unpack α,β = Params
    # Marginal revenue (must be positive)
    return max.(0.0, α .- β.*output)
end

################################################################################

# FOC for profit maximization
# Gives the solution of the foc as a function of output
function FOC_profit(output,Params)
    # First-order condition
    return MarginalRevenue(output,Params) - MarginalCost(output,Params)
end

# Function to be solved for
FOC(x) = FOC_profit(x,Params)

################################################################################
