######################################
# File with Functions and Structures #
######################################

# Structure for parameters
mutable struct ParamsStruc{T <: Real}
    # Grid of consumption
    cons::AbstractArray
    # Utility parameters
    γ::T                    # Parameter for exponential
    ρ::T                    # Parameter for CRRA
    α::T                    # Coefficients for quadratic terms
    β::T                    # Coefficients for quadratic terms
    # Utilities
    U_log::Vector{T}        # Logarithmic utility function
    U_exp::Vector{T}        # Exponential utility function
    U_CRRA::Vector{T}       # CRRA utility function
    U_quad::Vector{T}       # Quadratic utility function
end

################################################################################

# Set parameter values
function SetParams(; γ=1.0, ρ=1.5, α=0.5, β=0.2)
    # Grid for consumption going from .1 to 3 by increments of .05
    # This will be a vector of 59 points
    cons = range(0.1,step=0.05,stop=3.0)
    # Logarithmic utility function
    U_log = log.(cons)
    # Exponential utility function
    U_exp = -exp.(-γ.*cons)
    # CRRA utility function
    U_CRRA = (cons.^(1.0-ρ) .- 1.0)./(1.0-ρ)
    # Quadratic utility function
    U_quad = α.*cons .- β.*cons.^2.0./2.0
    # Output
    ParamsStruc(cons,γ,ρ,α,β,U_log,U_exp,U_CRRA,U_quad)
end

################################################################################

# Output table with key statistics
function OutputTable(Params)
    # Unpack objects
    @unpack cons,U_log,U_exp,U_CRRA,U_quad = Params
    # Table header
    table_header = ["Consumption", "Logarithmic", "Exponential", "CRRA", "Quadratic"]
    table_data = [cons U_log U_exp U_CRRA U_quad]
    # Print tables
    print("################################################################\n")
    pretty_table(table_data; header = table_header, tf = tf_markdown, formatters = ft_printf("%5.2f", 2:3) )
    # Save table as dataframe
    df = DataFrame("Consumption" => cons, "Logarithmic" => U_log, "Exponential" => U_exp,
                    "CRRA" => U_CRRA, "Quadratic" => U_quad)
    return df
end

################################################################################
